package domaine;

import outils.Liste;
import static outils.NomDeFichier.extension;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.LinkedList;

import domaine.fichiers.*;

public class ListeDAlarmes extends LinkedList<Alarme> implements Liste<Alarme>, Operable {
    private File fichier;
    private boolean modificationsApportees;
    public ListeDAlarmes() {
        super();
        modificationsApportees = false;
        fichier = null;
    }
    public boolean estVide() { return isEmpty(); }
    public int nombreDElements() { return size(); }
    public void ajoute(Alarme element) { add(element); trie(); }
    public void trie() { sort((Alarme a1, Alarme a2) -> a1.compareTo(a2)); }
    public void retire(int index) { remove(index); }
    public void retire(Alarme element) { remove(element); }
    public Alarme element(int index) { return get(index); }
    public void efface() { clear(); }
    public String nomAffichable() {
        String nom = (modificationsApportees) ? "*" : "";
        if (fichier == null) nom += "Sans nom";
        else nom += fichier.getName();
        return nom;
    }
    public boolean aEteModifiee() {
        return modificationsApportees;
    }
    public void modifiee() {
        modificationsApportees = true;
    }
    public void realise(Operation op) throws Exception {
        op.opereSur(this);
    }
    public ResultatEnregistrementChargement enregistreSous(String nomDeFichier) {
        fichier = new File(nomDeFichier);
        try {
        	Operation enregistrement;
        	TypeDeFichier type = TypeDeFichier.pour(extension(nomDeFichier));
        	switch (type) {
        		case TEXTE   : enregistrement = new EnregistrementTexte(nomDeFichier); break;
        		case XML     : enregistrement = new EnregistrementXML(nomDeFichier); break;
        		case BINAIRE : enregistrement = new EnregistrementBinaire(nomDeFichier); break;
        		default      : enregistrement = null;
        	}
			this.realise(enregistrement);
        } catch (FileNotFoundException ec) {
        	return ResultatEnregistrementChargement.ERREUR_DE_CREATION;
        } catch (VersionDeFichierNonGeree ev) {
        	return ResultatEnregistrementChargement.ERREUR_DE_VERSION;
    	} catch (Exception e) {
    		return ResultatEnregistrementChargement.ERREUR_D_ECRITURE;
    	}
        modificationsApportees = false;
        return ResultatEnregistrementChargement.REUSSI;
    }
    public ResultatEnregistrementChargement enregistre() { return enregistreSous(fichier.getAbsolutePath()); }
    public ResultatEnregistrementChargement charge(String nomDeFichier) {
        fichier = new File(nomDeFichier);
        try {
        	Operation chargement;
        	TypeDeFichier type = TypeDeFichier.pour(extension(nomDeFichier));
        	switch (type) {
        		case TEXTE   : chargement = new ChargementTexte(nomDeFichier); break;
        		case BINAIRE : chargement = new ChargementBinaire(nomDeFichier); break;
        		default      : chargement = null;
        	}
        	clear();
        	this.realise(chargement);
        } catch (FileNotFoundException eo) {
        	return ResultatEnregistrementChargement.FICHIER_INEXISTANT;
        } catch (VersionDeFichierNonGeree ev) {
        	return ResultatEnregistrementChargement.ERREUR_DE_VERSION;
    	} catch (Exception e) {
    		return ResultatEnregistrementChargement.ERREUR_DE_LECTURE;
    	}
        modificationsApportees = false;
        return ResultatEnregistrementChargement.REUSSI;
    }
}
