package domaine;

import static org.junit.jupiter.api.Assertions.*;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import domaine.fichiers.ResultatEnregistrementChargement;
import domaine.temps.MauvaisFormatDHeure;
import domaine.temps.MauvaiseValeurDesHeures;
import domaine.temps.MauvaiseValeurDesMinutes;

class TestEnregistrementEtChargementDeListeDAlarmes {
	private static ListeDAlarmes lst;
	private static ListeDAlarmes lue;
	@BeforeAll
	public static void initialiseListe() throws MauvaisFormatDHeure, MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
        lst = new ListeDAlarmes();
        lst.ajoute(new Alarme(8, 0));
        lst.ajoute(new Alarme(9, 0));
        lst.ajoute(new Alarme(12, 30));
        lst.ajoute(new Alarme(13, 30));
        lst.ajoute(new Alarme(17, 0));
        lst.ajoute(new Alarme(20, 0));
        lue = new ListeDAlarmes();
	}
    @Test
    public void test01EnregistrementEtChargementTexte() throws MauvaisFormatDHeure, MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
        assertEquals(ResultatEnregistrementChargement.REUSSI, lst.enregistreSous("test.txt"));
        assertEquals(ResultatEnregistrementChargement.REUSSI, lue.charge("test.txt"));
        assertEquals(lue.nombreDElements(), lst.nombreDElements());
        Iterator<Alarme> itlst = lst.iterator();
        Iterator<Alarme> itlue = lue.iterator();
        while(itlst.hasNext()) {
            assertEquals(0, itlst.next().compareTo(itlue.next()));
        }
    }
    @Test
    public void test02EnregistrementEtChargementBinaire() throws MauvaisFormatDHeure, MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
    	assertEquals(ResultatEnregistrementChargement.REUSSI, lst.enregistreSous("test.alrm"));
    	assertEquals(ResultatEnregistrementChargement.REUSSI, lue.charge("test.alrm"));
        assertEquals(lue.nombreDElements(), lst.nombreDElements());
        Iterator<Alarme> itlst = lst.iterator();
        Iterator<Alarme> itlue = lue.iterator();
        while(itlst.hasNext()) {
            assertEquals(0, itlst.next().compareTo(itlue.next()));
        }
    }
    @Test
    public void test03EnregistrementXML() throws MauvaisFormatDHeure, MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes, IOException {
        ListeDAlarmes lst;
        lst = new ListeDAlarmes();
        lst.ajoute(new Alarme(9, 0));
        lst.ajoute(new Alarme(15, 30));
        assertEquals(ResultatEnregistrementChargement.REUSSI, lst.enregistreSous("test.xml"));
    	BufferedReader fichier = new BufferedReader(new FileReader("test.xml"));
		assertEquals("<alarmes version=\"1\">", fichier.readLine());
		assertEquals("\t<alarme>", fichier.readLine());
		assertEquals("\t\t<heure>", fichier.readLine());
		assertEquals("\t\t\t09:00", fichier.readLine());
		assertEquals("\t\t</heure>", fichier.readLine());
		assertEquals("\t</alarme>", fichier.readLine());
		assertEquals("\t<alarme>", fichier.readLine());
		assertEquals("\t\t<heure>", fichier.readLine());
		assertEquals("\t\t\t15:30", fichier.readLine());
		assertEquals("\t\t</heure>", fichier.readLine());
		assertEquals("\t</alarme>", fichier.readLine());
		assertEquals("</alarmes>", fichier.readLine());
		fichier.close();
    }

}
